@@ -1,3 +1,5 @@
+#' @import stats utils
+
 servrEnv = new.env(parent = emptyenv())
 
 # turn file.info() to an HTML table
@@ -57,7 +59,7 @@ is_rstudio = function() Sys.getenv('RSTUDIO') == '1'
 # use the RStudio viewer if possible
 get_browser = function() {
   browser = if ('tools:rstudio' %in% search()) getOption('viewer') else {
-    if (is_rstudio()) getFromNamespace('viewer', 'rstudio')
+    if (is_rstudio()) getFromNamespace('viewer', 'rstudioapi')
   }
   # rstudio::viewer() does not seem to work when a separate R session is
   # launched from RStudio, so we need to try() and if it fails, fall back to the
@@ -78,7 +80,7 @@ in_dir = function(dir, expr) {
   expr
 }
 
-new_timeout = function(interval) {
+new_timeout = function(interval, max_timeout = 32) {
   old = NULL
   int = interval  # stores initial value (may need to restore later)
   function(delay = FALSE) {
@@ -87,7 +89,7 @@ new_timeout = function(interval) {
     # when delay = TRUE, wait for twice longer until returning TRUE (like Gmail)
     if (!is.na(delay)) {
       if (delay) {
-        int <<- 2 * int
+        int <<- min(2 * int, max_timeout)
         message('\n* Retrying in ', int, ' second', if (int > 1) 's', '...\n')
       } else int <<- interval
     }
